@@ -7,7 +7,7 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed', 'reftype', 'weaken';
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub meta { 
     require Class::MOP::Class;
@@ -30,9 +30,8 @@ sub new {
         
     (defined $name && $name)
         || confess "You must provide a name for the attribute";
-    (!exists $options{reader} && !exists $options{writer})
-        || confess "You cannot declare an accessor and reader and/or writer functions"
-            if exists $options{accessor};
+    $options{init_arg} = $name 
+        if not exists $options{init_arg};
             
     bless {
         name      => $name,
@@ -48,8 +47,27 @@ sub new {
     } => $class;
 }
 
+# NOTE:
+# this is a primative (and kludgy) clone operation 
+# for now, it will be repleace in the Class::MOP
+# bootstrap with a proper one, however we know 
+# that this one will work fine for now.
+sub clone {
+    my $self    = shift;
+    my %options = @_;
+    (blessed($self))
+        || confess "Can only clone an instance";
+    return bless { %{$self}, %options } => blessed($self);
+}
+
+# NOTE:
+# the next bunch of methods will get bootstrapped 
+# away in the Class::MOP bootstrapping section
+
 sub name { $_[0]->{name} }
 
+sub associated_class { $_[0]->{associated_class} }
+
 sub has_accessor  { defined($_[0]->{accessor})  ? 1 : 0 }
 sub has_reader    { defined($_[0]->{reader})    ? 1 : 0 }
 sub has_writer    { defined($_[0]->{writer})    ? 1 : 0 }
@@ -63,6 +81,9 @@ sub writer    { $_[0]->{writer}    }
 sub predicate { $_[0]->{predicate} }
 sub init_arg  { $_[0]->{init_arg}  }
 
+# end bootstrapped away method section.
+# (all methods below here are kept intact)
+
 sub default { 
     my $self = shift;
     if (reftype($self->{default}) && reftype($self->{default}) eq 'CODE') {
@@ -77,8 +98,6 @@ sub default {
 
 # class association 
 
-sub associated_class { $_[0]->{associated_class} }
-
 sub attach_to_class {
     my ($self, $class) = @_;
     (blessed($class) && $class->isa('Class::MOP::Class'))
@@ -245,6 +264,8 @@ An attribute must (at the very least), have a C<$name>. All other
 C<%options> are contained added as key-value pairs. Acceptable keys
 are as follows:
 
+=item B<clone (%options)>
+
 =over 4
 
 =item I<init_arg>
@@ -255,6 +276,9 @@ value of C<-foo>, then the following code will Just Work.
 
   MyClass->meta->construct_instance(-foo => "Hello There");
 
+In an init_arg is not assigned, it will automatically use the 
+value of C<$name>.
+
 =item I<default>
 
 The value of this key is the default value which 
